#! /bin/sh

# Script which grabs pics from input url
# Made by Hans-Christian Egtvedt <hc@egtvedt.no>
#
# Usage grabpics.sh [OPTIONS] [URL]
#
# Parameters
# -d, --delay		delay in seconds between each grab (30 default)
# -h, --help		prints this helpscreen
# -l, --log             turn on logging to file grabpics.log
# -o, --output-dir	directory to store the pictures
# -v, --verbose		be more verbose
#

# ===== END USER EDITING, NO EDITING BELOW THIS LINE! =====

# ===== ChangeLog =====
# v0.15 released - 26th September 2004
# * added storage in subdirs, <year>/<month>/<day>/
#
# v0.14 released - 25th September 2004
# * Bugfix with the --log option that did not log to file
#
# v0.13 released - 25th September 2004
# * Minor potential bugfixes
#
# v0.12 released - 25th September 2004
# * Added input check of [URL]
# * Added test of server
#
# v0.11 released - 25th September 2004
# * Fixed small typo in help information
#
# v0.10 released - 25th September 2004
# * Script mostly completed
# * Needs more input check of [URL] and --delay
#
# v0.01 released - 25th September 2004
# * Stared on script
#
# ===== END ChangeLog =====

# ===== START SCRIPT =====

# exit if anything returns nonzero
set -e

#parameters
version="0.15"
verdate="26/09 - 2004"
configured=1
printhelp=0
verbose=0
logging=0
s_delay=30
image_store_directory="images"

# print welcome greeting
echo "GrabPics v$version last updated $verdate"
echo "Created by Hans-Christian Egtvedt <hc@egtvedt.no>"

# print error when not configured
if [ "$configured" -eq 0 ]; then
	echo
	echo "Script not configured, exiting..."
	echo
	exit 1
fi

# read input parameteres
readargs=1;
readargsstop=$#;
while [ "$readargs" -le $readargsstop ]; do
	t_readargs=0
        if [ "$1" = "-v" -o "$1" = "--verbose" ]; then
                verbose=1
		t_readargs=1
        fi
        if [ "$1" = "-h" -o "$1" = "--help" ]; then
                printhelp=1
		t_readargs=1
        fi
        if [ "$1" = "-l" -o "$1" = "--log" ]; then
                logging=1
		t_readargs=1
        fi
        if [ "$1" = "-d" -o "$1" = "--delay" ]; then
		if [ -z $2 ]; then
			echo
			echo "$1 parameter needs a value"
			echo
			exit 1
		elif [ "$2" -le 0 ]; then
			echo
			echo "$1 is less than 1 or not a number"
			echo
			exit 1
		else
			shift 1
			readargs=$(($readargs+1))
			s_delay=$1
			t_readargs=1
		fi
        fi
        if [ "$1" = "-o" -o "$1" = "--output-dir" ]; then
		if [ -z $2 ]; then
			echo
			echo "$1 parameter needs a value"
			echo
			exit 1
		else
			shift 1
			readargs=$(($readargs+1))
        	        image_store_directory=$1
			t_readargs=1
		fi
        fi
	if [ $t_readargs -eq 0 ]; then
		graburl=$1
	fi

        shift 1;
        readargs=$(($readargs+1))
done

if [ $readargsstop -eq 0 ]; then
	echo
	echo "Please supply an [URL]"
	echo "grabpics.sh --help for mor information"
	echo
	exit 0
fi

# print help information
if [ "$printhelp" = "1" ]; then
        echo
        echo "Usage: grabpics [OPTIONS] [URL]"
        echo "Grab pictures from an url at regular intervals and store them with"
	echo "easy to read dateformat."
        echo
        echo "Mandatory arguments to long options are mandatory for short options too."
	echo "  -d, --delay           delay in seconds between each grab (30 default)"
	echo "  -h, --help            prints this helpscreen"
	echo "  -l, --log             turn on logging to file grabpics.log"
	echo "  -o, --output-dir      directory to store the pictures"
	echo "  -v, --verbose         be more verbose"
        echo
        echo "Report bugs to <hc@egtvedt.no>"
        echo
        exit 0
fi

set +e
graburl_num_http=`expr match "$graburl" "http"`
graburl_num_slash=`expr match "$graburl" "-"`
set -e

if [ -z $graburl -o $graburl_num_slash -eq 1 ]; then
	echo
	echo "Invalid URL supplied, exiting..."
	echo
	exit 1
fi

if ! wget --spider --quiet --timeout=5 --tries=10 "$graburl"; then
	echo
	echo "Url: \"$graburl\" is not valid, exiting..."
	echo
	exit 1
fi

if [ $verbose -eq 1 ]; then
	echo  "Check if "$image_store_directory\/" directory exist"
fi
if [ ! -d $image_store_directory ]; then
	if [ $verbose -eq 1 ]; then
	        echo  "Creating directory: $image_store_directory/"
	fi
	if ! mkdir -p $image_store_directory; then
		echo
		echo "Could not create "$image_store_directory\/", exiting..."
		echo
		exit 1
	fi
fi

if [ $verbose -eq 1 ]; then
	echo  "Check if mode on "$image_store_directory\/" is correct"
fi
if [ ! -w $image_store_directory ]; then
	if [ $verbose -eq 1 ]; then
	        echo  "Changing mode on "$image_store_directory\/" to 700"
	fi
	if ! chmod 700 $image_store_directory; then
		echo
		echo "Could not set mode on "$image_store_directory\/", exiting..."
		echo
		exit 1
	fi
fi

echo  "Starting grab of images from \"$graburl\""
while true; do
	img_filename=`date +img%Y%m%d%H%M%S.jpg`
	img_subdir=`date +%Y/%m/%d`
	if [ ! -d $img_subdir ]; then
		if mkdir -p $image_store_directory/$img_subdir; then
			if [ $verbose -eq 1 ]; then
				echo "New day, made subdir $image_store_directory/$img_subdir"
			fi
		else
			echo
			echo "Could not create subdir $image_store_directory/$img_subdir, exiting..."
			echo
			exit 1
		fi
	fi
	if [ $logging -eq 1 ]; then
		if ! wget --non-verbose --tries=5 -a "grabpics.log" -nv --connect-timeout=5 --limit-rate=200k -O "$image_store_directory/$img_subdir/$img_filename" "$graburl"; then
			rm -f "$image_store_directory/$img_subdir/$img_filename"
			if [ $verbose -eq 1 ]; then
				echo -n ":"
			fi
			echo
			echo "Failed fetching image, exiting..."
			echo
			exit 1
		fi
		if [ ! -s $image_store_directory/$img_subdir/$img_filename ]; then
			rm -f "$image_store_directory/$img_subdir/$img_filename"
			if [ $verbose -eq 1 ]; then
				echo -n ":"
			fi
		else
			if [ $verbose -eq 1 ]; then
				echo -n "."
			fi
		fi
	else
		if ! wget --quiet --tries=5 --connect-timeout=5 --limit-rate=200k -O "$image_store_directory/$img_subdir/$img_filename" "$graburl"; then
			rm -f "$image_store_directory/$img_subdir/$img_filename"
			if [ $verbose -eq 1 ]; then
				echo -n ":"
			fi
			echo
			echo "Failed fetching image, exiting..."
			echo
			exit 1
		fi
		if [ ! -s $image_store_directory/$img_subdir/$img_filename ]; then
			rm -f "$image_store_directory/$img_subdir/$img_filename"
			if [ $verbose -eq 1 ]; then
				echo -n ":"
			fi
		else
			if [ $verbose -eq 1 ]; then
				echo -n "."
			fi
		fi
	fi

	sleep $s_delay
done

exit 0
